@@ -1,6 +1,7 @@
 use crate::{
   cli::{
-    output_values_json, CreateTimeEntry, Format, StartTimeEntry, StopTimeEntry,
+    output_values_json, CreateTimeEntry, DeleteTimeEntry, Format,
+    StartTimeEntry, StopTimeEntry, TimeEntryDetails,
   },
   client::TogglClient,
   model::{Client, Project, Range, TimeEntry, Workspace},
@@ -16,6 +17,7 @@ use term_table::{
 };
 
 struct OutputEntry {
+  id: u64,
   date: NaiveDate,
   duration: Duration,
   workspace: String,
@@ -96,6 +98,7 @@ fn collect_output_entries(
     };
 
     output_entries.push(OutputEntry {
+      id: entry.id,
       date: entry.start.date().naive_local(),
       duration,
       workspace: maybe_workspace
@@ -197,7 +200,7 @@ pub(super) fn calculate_duration(
     let duration = end - start;
 
     if time_entry.lunch_break {
-      Ok(duration - launch_break())
+      calculate_duration_with_lunch_break(duration)
     } else {
       Ok(duration)
     }
@@ -208,6 +211,18 @@ pub(super) fn calculate_duration(
   }
 }
 
+fn calculate_duration_with_lunch_break(
+  duration: Duration,
+) -> anyhow::Result<Duration> {
+  let duration_with_lunch_break = duration - launch_break();
+
+  if duration_with_lunch_break <= Duration::zero() {
+    Err(anyhow!("Duration minus lunch break is <= 0"))
+  } else {
+    Ok(duration_with_lunch_break)
+  }
+}
+
 pub fn start(
   format: &Format,
   time_entry: &StartTimeEntry,
@@ -270,6 +285,34 @@ pub fn stop(
   Ok(())
 }
 
+pub fn delete(
+  format: &Format,
+  time_entry: &DeleteTimeEntry,
+  client: &TogglClient,
+) -> anyhow::Result<()> {
+  client.delete_time_entry(time_entry.id)?;
+
+  list(format, &Range::Today, client)?;
+
+  Ok(())
+}
+
+pub fn details(
+  format: &Format,
+  time_entry: &TimeEntryDetails,
+  client: &TogglClient,
+) -> anyhow::Result<()> {
+  let time_entry_details = client.time_entry_details(time_entry.id)?;
+
+  match format {
+    Format::Json => output_values_json(&[time_entry_details.data]),
+    Format::Raw => output_time_entry_raw(&time_entry_details.data),
+    Format::Table => output_time_entry_table(&time_entry_details.data),
+  }
+
+  Ok(())
+}
+
 fn output_time_entry_raw(time_entry: &TimeEntry) {
   println!(
     "{}\t{}\t{}\t{}",
@@ -313,9 +356,10 @@ fn output_values_raw(output_entries: &[OutputEntry]) {
     };
 
     println!(
-      "{}\t{}\t{}\t{}\t{}\t{}\t{}",
+      "{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}",
       &entry.date,
       duration_text,
+      &entry.id,
       &entry.workspace,
       &entry.project,
       &entry.client,
@@ -345,6 +389,7 @@ fn output_values_table(output_entries: &[OutputEntry]) {
     let header = Row::new(vec![
       TableCell::new("Date".bold().underline()),
       TableCell::new("Time".bold().underline()),
+      TableCell::new("Id".bold().underline()),
       TableCell::new("Workspace".bold().underline()),
       TableCell::new("Project".bold().underline()),
       TableCell::new("Customer".bold().underline()),
@@ -362,6 +407,7 @@ fn output_values_table(output_entries: &[OutputEntry]) {
       TableCell::new(""),
       TableCell::new(""),
       TableCell::new(""),
+      TableCell::new(""),
     ]));
 
     let mut total_time_sum = 0;
@@ -380,6 +426,7 @@ fn output_values_table(output_entries: &[OutputEntry]) {
         TableCell::new(""),
         TableCell::new(""),
         TableCell::new(""),
+        TableCell::new(""),
       ]);
 
       table.add_row(date_row);
@@ -394,6 +441,7 @@ fn output_values_table(output_entries: &[OutputEntry]) {
         let entry_row = Row::new(vec![
           TableCell::new(""),
           TableCell::new(duration_text),
+          TableCell::new(&entry.id),
           TableCell::new(&entry.workspace),
           TableCell::new(&entry.project),
           TableCell::new(&entry.client),
